// Code generated by "stringer -type edgeKind"; DO NOT EDIT.

package unused

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[edgeAlias-1]
	_ = x[edgeBlankField-2]
	_ = x[edgeAnonymousStruct-4]
	_ = x[edgeCgoExported-8]
	_ = x[edgeConstGroup-16]
	_ = x[edgeElementType-32]
	_ = x[edgeEmbeddedInterface-64]
	_ = x[edgeExportedConstant-128]
	_ = x[edgeExportedField-256]
	_ = x[edgeExportedFunction-512]
	_ = x[edgeExportedMethod-1024]
	_ = x[edgeExportedType-2048]
	_ = x[edgeExportedVariable-4096]
	_ = x[edgeExtendsExportedFields-8192]
	_ = x[edgeExtendsExportedMethodSet-16384]
	_ = x[edgeFieldAccess-32768]
	_ = x[edgeFunctionArgument-65536]
	_ = x[edgeFunctionResult-131072]
	_ = x[edgeFunctionSignature-262144]
	_ = x[edgeImplements-524288]
	_ = x[edgeInstructionOperand-1048576]
	_ = x[edgeInterfaceCall-2097152]
	_ = x[edgeInterfaceMethod-4194304]
	_ = x[edgeKeyType-8388608]
	_ = x[edgeLinkname-16777216]
	_ = x[edgeMainFunction-33554432]
	_ = x[edgeNamedType-67108864]
	_ = x[edgeNetRPCRegister-134217728]
	_ = x[edgeNoCopySentinel-268435456]
	_ = x[edgeProvidesMethod-536870912]
	_ = x[edgeReceiver-1073741824]
	_ = x[edgeRuntimeFunction-2147483648]
	_ = x[edgeSignature-4294967296]
	_ = x[edgeStructConversion-8589934592]
	_ = x[edgeTestSink-17179869184]
	_ = x[edgeTupleElement-34359738368]
	_ = x[edgeType-68719476736]
	_ = x[edgeTypeName-137438953472]
	_ = x[edgeUnderlyingType-274877906944]
	_ = x[edgePointerType-549755813888]
	_ = x[edgeUnsafeConversion-1099511627776]
	_ = x[edgeUsedConstant-2199023255552]
	_ = x[edgeVarDecl-4398046511104]
}

const _edgeKind_name = "edgeAliasedgeBlankFieldedgeAnonymousStructedgeCgoExportededgeConstGroupedgeElementTypeedgeEmbeddedInterfaceedgeExportedConstantedgeExportedFieldedgeExportedFunctionedgeExportedMethodedgeExportedTypeedgeExportedVariableedgeExtendsExportedFieldsedgeExtendsExportedMethodSetedgeFieldAccessedgeFunctionArgumentedgeFunctionResultedgeFunctionSignatureedgeImplementsedgeInstructionOperandedgeInterfaceCalledgeInterfaceMethodedgeKeyTypeedgeLinknameedgeMainFunctionedgeNamedTypeedgeNetRPCRegisteredgeNoCopySentineledgeProvidesMethodedgeReceiveredgeRuntimeFunctionedgeSignatureedgeStructConversionedgeTestSinkedgeTupleElementedgeTypeedgeTypeNameedgeUnderlyingTypeedgePointerTypeedgeUnsafeConversionedgeUsedConstantedgeVarDecl"

var _edgeKind_map = map[edgeKind]string{
	1:             _edgeKind_name[0:9],
	2:             _edgeKind_name[9:23],
	4:             _edgeKind_name[23:42],
	8:             _edgeKind_name[42:57],
	16:            _edgeKind_name[57:71],
	32:            _edgeKind_name[71:86],
	64:            _edgeKind_name[86:107],
	128:           _edgeKind_name[107:127],
	256:           _edgeKind_name[127:144],
	512:           _edgeKind_name[144:164],
	1024:          _edgeKind_name[164:182],
	2048:          _edgeKind_name[182:198],
	4096:          _edgeKind_name[198:218],
	8192:          _edgeKind_name[218:243],
	16384:         _edgeKind_name[243:271],
	32768:         _edgeKind_name[271:286],
	65536:         _edgeKind_name[286:306],
	131072:        _edgeKind_name[306:324],
	262144:        _edgeKind_name[324:345],
	524288:        _edgeKind_name[345:359],
	1048576:       _edgeKind_name[359:381],
	2097152:       _edgeKind_name[381:398],
	4194304:       _edgeKind_name[398:417],
	8388608:       _edgeKind_name[417:428],
	16777216:      _edgeKind_name[428:440],
	33554432:      _edgeKind_name[440:456],
	67108864:      _edgeKind_name[456:469],
	134217728:     _edgeKind_name[469:487],
	268435456:     _edgeKind_name[487:505],
	536870912:     _edgeKind_name[505:523],
	1073741824:    _edgeKind_name[523:535],
	2147483648:    _edgeKind_name[535:554],
	4294967296:    _edgeKind_name[554:567],
	8589934592:    _edgeKind_name[567:587],
	17179869184:   _edgeKind_name[587:599],
	34359738368:   _edgeKind_name[599:615],
	68719476736:   _edgeKind_name[615:623],
	137438953472:  _edgeKind_name[623:635],
	274877906944:  _edgeKind_name[635:653],
	549755813888:  _edgeKind_name[653:668],
	1099511627776: _edgeKind_name[668:688],
	2199023255552: _edgeKind_name[688:704],
	4398046511104: _edgeKind_name[704:715],
}

func (i edgeKind) String() string {
	if str, ok := _edgeKind_map[i]; ok {
		return str
	}
	return "edgeKind(" + strconv.FormatInt(int64(i), 10) + ")"
}
